@@ -1,6 +1,6 @@
 use std::net::Ipv4Addr;
 
-use crate::dhcp::{DhcpMessageTypes, DhcpOption, DhcpPacket};
+use crate::dhcp::{ClientFqdn, DhcpMessageTypes, DhcpOption, DhcpPacket};
 use crate::{
     args,
     dhcp::{self, DhcpOptions},
@@ -33,9 +33,6 @@ pub fn main(args: args::discover::Arguments) -> Result<(), String> {
         false => dhcp_mac,
     };
 
-    let mut channel = TransportChannel::new(iface, Some(args.timeout))
-        .map_err(|e| format!("Unable to create a raw socket: {}", e))?;
-
     let request_list = match args.options {
         Some(options) => options,
         None => vec![
@@ -47,12 +44,35 @@ pub fn main(args: args::discover::Arguments) -> Result<(), String> {
         ],
     };
 
+    let dns_update = args.dns_update;
+    let fqdn = args.fqdn.unwrap_or("".to_string());
+
+    let client_fqdn = if dns_update || fqdn.len() != 0 {
+        Some(ClientFqdn {
+            flags: dns_update as u8,
+            aresult: 0,
+            ptrresult: 0,
+            name: fqdn,
+        })
+    } else {
+        None
+    };
+
+    let client_options = ClientOptions {
+        request_list,
+        hostname: args.hostname,
+        client_fqdn,
+    };
+
+    let mut channel = TransportChannel::new(iface, Some(args.timeout))
+        .map_err(|e| format!("Unable to create a raw socket: {}", e))?;
+
     match get_action(args.inform, args.send_request) {
         Action::Discover => discover_all(
             &mut channel,
             dhcp_mac,
             ether_mac,
-            request_list,
+            &client_options,
             args.servers.as_ref(),
         )?,
         Action::DiscoverAndRequest => {
@@ -60,7 +80,7 @@ pub fn main(args: args::discover::Arguments) -> Result<(), String> {
                 &mut channel,
                 dhcp_mac,
                 ether_mac,
-                request_list,
+                &client_options,
                 args.servers.as_ref(),
             )?;
             print_response(&resp);
@@ -77,7 +97,7 @@ pub fn main(args: args::discover::Arguments) -> Result<(), String> {
                 my_ip,
                 dhcp_mac,
                 ether_mac,
-                request_list,
+                &client_options,
                 args.servers.as_ref(),
             )?;
         }
@@ -102,14 +122,20 @@ fn discover_all(
     channel: &mut TransportChannel,
     dhcp_mac: MacAddr,
     ether_mac: MacAddr,
-    request_list: Vec<u8>,
+    client_options: &ClientOptions,
     servers: Option<&Vec<Ipv4Addr>>,
 ) -> Result<(), String> {
     let mut rng = rand::thread_rng();
     let transaction_id: u32 = rng.gen();
 
     info!("DISCOVER sent - Client MAC {}", dhcp_mac);
-    send_discover(channel, transaction_id, dhcp_mac, ether_mac, request_list)?;
+    send_discover(
+        channel,
+        transaction_id,
+        dhcp_mac,
+        ether_mac,
+        client_options,
+    )?;
 
     loop {
         match channel.recv_dhcp(
@@ -137,7 +163,7 @@ fn inform_all(
     client_ip: Ipv4Addr,
     dhcp_mac: MacAddr,
     ether_mac: MacAddr,
-    request_list: Vec<u8>,
+    client_options: &ClientOptions,
     servers: Option<&Vec<Ipv4Addr>>,
 ) -> Result<(), String> {
     let mut rng = rand::thread_rng();
@@ -150,7 +176,7 @@ fn inform_all(
         transaction_id,
         dhcp_mac,
         ether_mac,
-        request_list,
+        client_options,
         client_ip,
     )?;
 
@@ -229,7 +255,7 @@ fn discover_and_request(
     channel: &mut TransportChannel,
     dhcp_mac: MacAddr,
     ether_mac: MacAddr,
-    request_list: Vec<u8>,
+    client_options: &ClientOptions,
     servers: Option<&Vec<Ipv4Addr>>,
 ) -> Result<DhcpPacket, String> {
     let mut rng = rand::thread_rng();
@@ -241,7 +267,7 @@ fn discover_and_request(
         transaction_id,
         dhcp_mac,
         ether_mac,
-        request_list.clone(),
+        client_options,
         servers,
     )?;
 
@@ -258,7 +284,7 @@ fn discover_and_request(
         transaction_id,
         dhcp_mac,
         ether_mac,
-        request_list,
+        client_options,
         client_ip,
         dhcp_server,
     );
@@ -269,10 +295,16 @@ fn send_recv_discover(
     transaction_id: u32,
     dhcp_mac: MacAddr,
     ether_mac: MacAddr,
-    request_list: Vec<u8>,
+    client_options: &ClientOptions,
     servers: Option<&Vec<Ipv4Addr>>,
 ) -> Result<DhcpPacket, String> {
-    send_discover(channel, transaction_id, dhcp_mac, ether_mac, request_list)?;
+    send_discover(
+        channel,
+        transaction_id,
+        dhcp_mac,
+        ether_mac,
+        client_options,
+    )?;
 
     let (_, dhcp_resp) = channel.recv_dhcp(
         dhcp::DHCP_CLIENT_PORT,
@@ -289,13 +321,13 @@ fn send_discover(
     transaction_id: u32,
     dhcp_mac: MacAddr,
     ether_mac: MacAddr,
-    request_list: Vec<u8>,
+    client_options: &ClientOptions,
 ) -> Result<(), String> {
     let mut dp = DhcpPacket::new_request();
     dp.add_dhcp_msg_type(DhcpMessageTypes::DISCOVER);
     dp.xid = transaction_id;
     dp.chaddr = dhcp_mac;
-    dp.add_parameter_request_list(request_list);
+    set_client_options(&mut dp, client_options);
 
     channel
         .build_and_send(
@@ -318,7 +350,7 @@ fn send_recv_request(
     transaction_id: u32,
     dhcp_mac: MacAddr,
     ether_mac: MacAddr,
-    request_list: Vec<u8>,
+    client_options: &ClientOptions,
     client_ip: Ipv4Addr,
     dhcp_server: Ipv4Addr,
 ) -> Result<DhcpPacket, String> {
@@ -328,7 +360,8 @@ fn send_recv_request(
     dp.chaddr = dhcp_mac;
     dp.add_requested_ip_address(client_ip);
     dp.add_dhcp_server_id(dhcp_server);
-    dp.add_parameter_request_list(request_list);
+
+    set_client_options(&mut dp, client_options);
 
     channel
         .build_and_send(
@@ -358,7 +391,7 @@ fn send_inform(
     transaction_id: u32,
     dhcp_mac: MacAddr,
     ether_mac: MacAddr,
-    request_list: Vec<u8>,
+    client_options: &ClientOptions,
     client_ip: Ipv4Addr,
 ) -> Result<(), String> {
     let mut dp = DhcpPacket::new_request();
@@ -366,7 +399,7 @@ fn send_inform(
     dp.xid = transaction_id;
     dp.ciaddr = client_ip;
     dp.chaddr = dhcp_mac;
-    dp.add_parameter_request_list(request_list.clone());
+    set_client_options(&mut dp, client_options);
 
     channel
         .build_and_send(
@@ -383,3 +416,21 @@ fn send_inform(
 
     return Ok(());
 }
+
+pub fn set_client_options(dp: &mut DhcpPacket, co: &ClientOptions) {
+    dp.add_parameter_request_list(co.request_list.clone());
+
+    if let Some(hostname) = &co.hostname {
+        dp.add_hostname(hostname.clone());
+    }
+
+    if let Some(client_fqdn) = &co.client_fqdn {
+        dp.add_client_fqdn(client_fqdn.clone());
+    }
+}
+
+pub struct ClientOptions {
+    pub request_list: Vec<u8>,
+    pub hostname: Option<String>,
+    pub client_fqdn: Option<ClientFqdn>,
+}
